commit 8e4dc284e2f576d77cccb65bdcd501c0a9e3b21f
Author: root <root@nuedel.(none)>
Date:   Sun Jun 14 12:07:10 2009 +0200

    Adding IMSI to caller info.
    This allows to identify IMSI by application, as well as dialing IMSI.

diff --git a/openbsc/include/openbsc/mncc.h b/openbsc/include/openbsc/mncc.h
index c04a81f..07d2bdb 100644
--- a/openbsc/include/openbsc/mncc.h
+++ b/openbsc/include/openbsc/mncc.h
@@ -123,6 +123,7 @@ struct gsm_mncc_number {
 	int		present;
 	int		screen;
 	char		number[33];
+	char		imsi[16];
 };
 
 struct gsm_mncc_cause {
diff --git a/openbsc/src/gsm_04_08.c b/openbsc/src/gsm_04_08.c
index 1e432de..1588ec6 100644
--- a/openbsc/src/gsm_04_08.c
+++ b/openbsc/src/gsm_04_08.c
@@ -2198,6 +2198,8 @@ static int gsm48_cc_rx_setup(struct gsm_trans *trans, struct msgb *msg)
 		setup.fields |= MNCC_F_CALLING;
 		strncpy(setup.calling.number, trans->subscr->extension,
 			sizeof(setup.calling.number)-1);
+		strncpy(setup.calling.imsi, trans->subscr->imsi,
+			sizeof(setup.calling.imsi)-1);
 	}
 	/* bearer capability */
 	if (TLVP_PRESENT(&tp, GSM48_IE_BEARER_CAP)) {
@@ -2534,6 +2536,8 @@ static int gsm48_cc_rx_connect(struct gsm_trans *trans, struct msgb *msg)
 		connect.fields |= MNCC_F_CONNECTED;
 		strncpy(connect.connected.number, trans->subscr->extension,
 			sizeof(connect.connected.number)-1);
+		strncpy(connect.connected.imsi, trans->subscr->imsi,
+			sizeof(connect.connected.imsi)-1);
 	}
 	/* facility */
 	if (TLVP_PRESENT(&tp, GSM48_IE_FACILITY)) {
@@ -3373,8 +3377,18 @@ int mncc_send(struct gsm_network *net, int msg_type, void *arg)
 			/* Invalid call reference */
 			return mncc_release_ind(net, NULL, data->callref, 1, 81);
 		}
+		if (!data->called.number[0] && !data->called.imsi[0]) {
+			DEBUGP(DCC, "(bts - trx - ts - ti) "
+				"Received '%s' from MNCC with "
+				"no number or IMSI\n", get_mncc_name(msg_type));
+			/* Invalid number */
+			return mncc_release_ind(net, NULL, data->callref, 1, 28);
+		}
 		/* New transaction due to setup, find subscriber */
-		subscr = subscr_get_by_extension(data->called.number);
+		if (data->called.number[0])
+			subscr = subscr_get_by_extension(data->called.number);
+		else
+			subscr = subscr_get_by_imsi(data->called.imsi);
 		/* If subscriber is not found */
 		if (!subscr) {
 			DEBUGP(DCC, "(bts - trx - ts - ti -- sub %s) "
